@@ -3,7 +3,7 @@ from . import multipart_collector, cookie, session, grpc_collector, ws_executor
 from . import wsgi_executor, xmlrpc_executor, grpc_executor, jsonrpc_executor
 from aquests.protocols.grpc import discover
 from sqlphile import Template, DB_PGSQL
-from skitai.wsgi_apps import Config
+from skitai.wastuff.wsgi_apps import Config
 import skitai
 import sys
 from .app import mixin
@@ -19,9 +19,20 @@ class Atila (mixin.MixIn):
         self.sqlphile = None
         # for bus, set by wsgi_executor
         self.config = Config (preset = True)
+        self.set_defulat_config ()
+        self.config.MINIFY_HTML = False
         self._aliases = []
         self._sqlmap_dir = None
 
+    def set_defulat_config (self):
+        from . import version_info
+
+        self.config.MINIFY_HTML = False
+        if version_info [:2] >= (0, 8):
+            self.config.JSON_ENCODER = 'utcoffset'
+        else:
+            self.config.JSON_ENCODER = 'utc'
+
     def alias (self, *args, **karg):
         name, args = skitai.alias (*args, **karg)
         skitai.dconf ["clusters"].pop (name)
