@@ -1,7 +1,7 @@
 import os, sys
 import time
 from hmac import new as hmac
-from hashlib import sha1, sha256
+from hashlib import sha1, sha256, md5
 import base64
 import pickle
 import random
@@ -34,19 +34,27 @@ class WAS (wsgiappservice.WAS):
         return self.apps.build_url (thing, *args, **karg)
     ab = urlfor
 
-    def partial (self, thing, **karg):
+    def urlpatch (self, thing, **karg):
         # override with current args
-        karg ["__defaults__"] = self.request.args
-        return self.ab (thing, **karg)
+        defaults = self.request.PARAMS
+        defaults.update (self.request.URL)
+        karg ["__defaults__"] = defaults
+        return self.urlfor (thing, **karg)
+    partial = urlpatch
 
     def baseurl (self, thing):
         # resource path info without parameters
-        return self.ab (thing, __resource_path_only__ = True)
+        return self.urlfor (thing, __resource_path_only__ = True)
     basepath = baseurl
 
+    def urlspec (self, thing):
+        # resource path info without parameters
+        return self.urlfor (thing, __resource_spec_only__ = True)
+
     # event -------------------------------------------------
     def broadcast (self, event, *args, **kargs):
         return self.apps.bus.emit (event, self, *args, **kargs)
+    emit = broadcast
 
     # passowrd en/decrypt -----------------------------------
     def encrypt_password (self, password):
@@ -60,6 +68,9 @@ class WAS (wsgiappservice.WAS):
         signature_ = base64.b64encode (dig).decode ()
         return signature == signature_
 
+    def make_uid (self, s):
+        return base64.encodestring (md5 (s.encode ()).digest ()) [:-3].decode ().replace ('/', '-').replace ('+', '_')
+
     # JWT token --------------------------------------------------
     def mkjwt (self, claim, salt = None, alg = "HS256"):
         assert "exp" in claim, "exp claim required"
@@ -130,8 +141,9 @@ class WAS (wsgiappservice.WAS):
                 del self.session [tokey]
                 self.session.mount ()
             return
+        elif tokey:
+            self.session.mount ()
 
-        self.session.mount ()
         obj = wrapper ['object']
         return obj
 
@@ -143,6 +155,7 @@ class WAS (wsgiappservice.WAS):
             return
         tokey, token = session_token
         self.session.mount (tokey [1:-6])
+
         if not self.session.get (tokey):
             self.session.mount ()
             return
@@ -158,8 +171,9 @@ class WAS (wsgiappservice.WAS):
     CSRF_NAME = "_csrf_token"
     @property
     def csrf_token (self):
+        self.session.mount ()
         if self.CSRF_NAME not in self.session:
-            self.session [self.CSRF_NAME] = hex (random.getrandbits (64))
+            self.session [self.CSRF_NAME] = hex (random.getrandbits (64)) [2:]
         return self.session [self.CSRF_NAME]
 
     @property
@@ -172,6 +186,7 @@ class WAS (wsgiappservice.WAS):
         token = self.request.args [self.CSRF_NAME]
         if self.csrf_token == token:
             if not keep:
+                self.session.mount ()
                 del self.session [self.CSRF_NAME]
             return True
         return False
@@ -198,11 +213,22 @@ class WAS (wsgiappservice.WAS):
     def render (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
         return self.app.render (self, template_file, _do_not_use_this_variable_name_, **karg)
 
+    def render_or_API (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
+        if self.request.acceptable ('application/json'):
+            return self.API (None, _do_not_use_this_variable_name_, **karg)
+        return self.render (template_file, _do_not_use_this_variable_name_ = {}, **karg)
+
     def API (self, *args, **karg):
-        return self.response.API (*args, **karg)
+        api = self.response.API (*args, **karg)
+        try: api.set_json_encoder (self.app.config.get ("JSON_ENCODER"))
+        except AttributeError: pass
+        return api
 
     def Fault (self, *args, **karg):
-        return self.response.Fault (*args, **karg)
+        fault = self.response.Fault (*args, **karg)
+        try: fault.set_json_encoder (self.app.config.get ("JSON_ENCODER"))
+        except AttributeError: pass
+        return fault
 
     @property
     def Error (self):
